@@ -23,7 +23,6 @@
 #include "highlight-effect.h"
 
 #include <QAbstractItemView>
-#include <QAbstractButton>
 #include <QMenu>
 #include <QStyleOption>
 
@@ -37,82 +36,66 @@
 
 #include <QDebug>
 
-#define TORLERANCE 40
+#define ColorDifference 10
 
-static QColor symbolic_color = Qt::gray;
+QColor HighLightEffect::symbolic_color = QColor(31, 32, 34, 192);
 
 void HighLightEffect::setSkipEffect(QWidget *w, bool skip)
 {
     w->setProperty("skipHighlightIconEffect", skip);
 }
 
+
+
 bool HighLightEffect::isPixmapPureColor(const QPixmap &pixmap)
 {
-    QImage img = pixmap.toImage();
-    bool init = false;
-    int red = 0;
-    int green = 0;
-    int blue = 0;
-    qreal variance = 0;
-    qreal mean = 0;
-    qreal standardDeviation = 0;
-    QVector<int> pixels;
-    bool isPure = true;
-    bool isFullyPure = true;
-    for (int x = 0; x < img.width(); x++) {
-        for (int y = 0; y < img.height(); y++) {
-            auto color = img.pixelColor(x, y);
-            if (color.alpha() != 0) {
-                int hue = color.hue();
-                pixels<<hue;
-                mean += hue;
-                if (!init) {
-                    red = color.red();
-                    green = color.green();
-                    blue = color.blue();
-                    init = true;
-                } else {
-                    color.setAlpha(255);
-                    int r = color.red();
-                    int g = color.green();
-                    int b = color.blue();
-                    int dr = qAbs(r - red);
-                    int dg = qAbs(g - green);
-                    int db = qAbs(b - blue);
-                    bool same = dr < TORLERANCE && dg < TORLERANCE && db < TORLERANCE;
-                    if (isFullyPure) {
-                        if (dr > 0 || dg > 0 || db > 0) {
-                            isFullyPure = false;
-                        }
-                    }
-                    if (!same) {
-                        if (isPure || isFullyPure) {
-                            isPure = false;
-                            isFullyPure = false;
-                            break;
-                        }
-                    }
-                }
+    QImage image = pixmap.toImage();
+
+    QVector<QColor> vector;
+    int total_red = 0;
+    int total_green = 0;
+    int total_blue = 0;
+    bool pure = true;
+    for (int y = 0; y < image.height(); ++y) {
+        for (int x = 0; x < image.width(); ++x) {
+            if (image.pixelColor(x, y).alphaF() > 0.3) {
+                QColor color = image.pixelColor(x, y);
+                vector << color;
+                total_red += color.red();
+                total_green += color.green();
+                total_blue += color.blue();
+                int dr = qAbs(color.red() - symbolic_color.red());
+                int dg = qAbs(color.green() - symbolic_color.green());
+                int db = qAbs(color.blue() - symbolic_color.blue());
+                if (dr > ColorDifference || dg > ColorDifference || db > ColorDifference)
+                    pure = false;
             }
         }
     }
 
-    if (isPure)
+    if (pure)
         return true;
 
-    mean = mean/pixels.count();
-    for (auto hue : pixels) {
-        variance += (hue - mean)*(hue - mean);
+    qreal squareRoot_red = 0;
+    qreal squareRoot_green = 0;
+    qreal squareRoot_blue = 0;
+    qreal average_red = total_red / vector.count();
+    qreal average_green = total_green / vector.count();
+    qreal average_blue = total_blue / vector.count();
+    for (QColor color : vector) {
+        squareRoot_red += (color.red() - average_red) * (color.red() - average_red);
+        squareRoot_green += (color.green() - average_green) * (color.green() - average_green);
+        squareRoot_blue += (color.blue() - average_blue) * (color.blue() - average_blue);
     }
 
-    standardDeviation = qSqrt(variance/pixels.count());
-
-    isFullyPure = standardDeviation == 0 || variance == 0;
-    isPure = standardDeviation < 1 || variance == 0;
-
-    return isPure;
+    qreal arithmeticSquareRoot_red = qSqrt(squareRoot_red / vector.count());
+    qreal arithmeticSquareRoot_green = qSqrt(squareRoot_green / vector.count());
+    qreal arithmeticSquareRoot_blue = qSqrt(squareRoot_blue / vector.count());
+    return arithmeticSquareRoot_red < 2.0 && arithmeticSquareRoot_green < 2.0 && arithmeticSquareRoot_blue < 2.0;
 }
 
+
+
 bool HighLightEffect::setMenuIconHighlightEffect(QMenu *menu, HighLightMode hlmode, EffectMode mode)
 {
     if (!menu)
@@ -133,16 +116,6 @@ bool HighLightEffect::setViewItemIconHighlightEffect(QAbstractItemView *view, Hi
     return true;
 }
 
-bool HighLightEffect::setButtonIconHighlightEffect(QAbstractButton *button, HighLightMode hlmode, EffectMode mode)
-{
-    if (!button)
-        return false;
-
-    button->setProperty("useIconHighlightEffect", hlmode);
-    button->setProperty("iconHighlightEffectMode", mode);
-    return true;
-}
-
 HighLightEffect::HighLightMode HighLightEffect::isWidgetIconUseHighlightEffect(const QWidget *w)
 {
     if (w) {
@@ -214,8 +187,7 @@ QPixmap HighLightEffect::generatePixmap(const QPixmap &pixmap, const QStyleOptio
                 option->state & QStyle::State_Sunken) {
             p.fillRect(target.rect(), option->palette.highlightedText());
         } else {
-            if (mode == BothDefaultAndHighlit)
-                p.fillRect(target.rect(), defaultStyleDark());
+                p.fillRect(target.rect(), mode ? option->palette.text() : defaultStyleDark());
         }
         return target;
     }
@@ -232,7 +204,7 @@ QPixmap HighLightEffect::generatePixmap(const QPixmap &pixmap, const QStyleOptio
     }
 
     if (widget->property("iconHighlightEffectMode").isValid()) {
-        mode = EffectMode(widget->property("iconHighlightEffectMode").toInt());
+        mode = EffectMode(widget->property("iconHighlightEffectMode").toBool());
     }
 
     HighLightMode hlmode = isWidgetIconUseHighlightEffect(widget);
@@ -250,11 +222,6 @@ QPixmap HighLightEffect::generatePixmap(const QPixmap &pixmap, const QStyleOptio
                 option->state.testFlag(QStyle::State_Sunken) ||
                 option->state.testFlag(QStyle::State_On) ||
                 option->state.testFlag(QStyle::State_Selected);
-        if (auto button = qobject_cast<const QAbstractButton *>(widget)) {
-            if (button->isDown() || button->isChecked()) {
-                overOrDown = true;
-            }
-        }
 
         if (qobject_cast<const QAbstractItemView *>(widget)) {
             if (!option->state.testFlag(QStyle::State_Selected))
@@ -276,24 +243,19 @@ QPixmap HighLightEffect::generatePixmap(const QPixmap &pixmap, const QStyleOptio
             p.fillRect(target.rect(), option->palette.highlightedText());
             return target;
         } else {
-            if (mode == BothDefaultAndHighlit) {
-                QPixmap target = pixmap;
-
-                if (fillIconSymbolicColor) {
-                    target = filledSymbolicColoredPixmap(pixmap, option->palette.highlightedText().color());
-                }
-
-                if (!isPurePixmap)
-                    return target;
-
-                QPainter p(&target);
-                p.setRenderHint(QPainter::Antialiasing);
-                p.setRenderHint(QPainter::SmoothPixmapTransform);
-                p.setCompositionMode(QPainter::CompositionMode_SourceIn);
-                p.fillRect(target.rect(), defaultStyleDark());
-                //p.end();
-                return target;
+            QPixmap target = pixmap;
+            if (fillIconSymbolicColor) {
+                target = filledSymbolicColoredPixmap(pixmap, option->palette.highlightedText().color());
             }
+            if (!isPurePixmap)
+                return target;
+
+            QPainter p(&target);
+            p.setRenderHint(QPainter::Antialiasing);
+            p.setRenderHint(QPainter::SmoothPixmapTransform);
+            p.setCompositionMode(QPainter::CompositionMode_SourceIn);
+            p.fillRect(target.rect(), mode ? option->palette.text() : defaultStyleDark());
+            return target;
         }
     } else if (hlmode == ordinaryHighLight) {
         return ordinaryGeneratePixmap(pixmap, option, widget, mode);
@@ -314,9 +276,11 @@ QPixmap HighLightEffect::generatePixmap(const QPixmap &pixmap, const QStyleOptio
     return pixmap;
 }
 
-QPixmap HighLightEffect:: ordinaryGeneratePixmap(const QPixmap &pixmap, const QStyleOption *option, const QWidget *widget, EffectMode mode)
+QPixmap HighLightEffect::ordinaryGeneratePixmap(const QPixmap &pixmap, const QStyleOption *option, const QWidget *widget, EffectMode mode)
 {
-    if (!isPixmapPureColor(pixmap))
+    if (pixmap.isNull())
+        return pixmap;
+    if (!isPixmapPureColor(pixmap) || !(option->state & QStyle::State_Enabled))
         return pixmap;
 
     QPixmap target = pixmap;
@@ -326,20 +290,22 @@ QPixmap HighLightEffect:: ordinaryGeneratePixmap(const QPixmap &pixmap, const QS
     }
 
     if (widget && widget->property("iconHighlightEffectMode").isValid()) {
-        mode = EffectMode(widget->property("iconHighlightEffectMode").toInt());
+        mode = EffectMode(widget->property("iconHighlightEffectMode").toBool());
     }
 
     QPainter p(&target);
     p.setRenderHint(QPainter::Antialiasing);
     p.setRenderHint(QPainter::SmoothPixmapTransform);
     p.setCompositionMode(QPainter::CompositionMode_SourceIn);
-    p.fillRect(target.rect(), color.isValid() ? color : (mode ? defaultStyleDark() : option->palette.text()));
+    p.fillRect(target.rect(), color.isValid() ? color : (mode ? option->palette.text() : defaultStyleDark()));
     return target;
 }
 
-QPixmap HighLightEffect::hoverGeneratePixmap(const QPixmap &pixmap, const QStyleOption *option, const QWidget *widget)
+QPixmap HighLightEffect::hoverGeneratePixmap(const QPixmap &pixmap, const QStyleOption *option, const QWidget *widget, EffectMode mode)
 {
-    if (!isPixmapPureColor(pixmap))
+    if (pixmap.isNull())
+        return pixmap;
+    if (!isPixmapPureColor(pixmap) || !(option->state & QStyle::State_Enabled))
         return pixmap;
 
     QPixmap target = pixmap;
@@ -348,22 +314,21 @@ QPixmap HighLightEffect::hoverGeneratePixmap(const QPixmap &pixmap, const QStyle
         color = widget->property("setIconHighlightEffectHoverColor").value<QColor>();
     }
 
-    bool isEnable = option->state.testFlag(QStyle::State_Enabled);
+    if (widget && widget->property("iconHighlightEffectMode").isValid()) {
+        mode = EffectMode(widget->property("iconHighlightEffectMode").toBool());
+    }
+
     bool overOrDown =  option->state.testFlag(QStyle::State_MouseOver) ||
             option->state.testFlag(QStyle::State_Sunken) ||
             option->state.testFlag(QStyle::State_On) ||
             option->state.testFlag(QStyle::State_Selected);
-    if (auto button = qobject_cast<const QAbstractButton *>(widget)) {
-        if (button->isDown() || button->isChecked())
-            overOrDown = true;
-    }
     if (qobject_cast<const QAbstractItemView *>(widget)) {
         if (!option->state.testFlag(QStyle::State_Selected))
             overOrDown = false;
     }
 
     QPainter p(&target);
-    if (isEnable && overOrDown) {
+    if (overOrDown) {
         p.setRenderHint(QPainter::Antialiasing);
         p.setRenderHint(QPainter::SmoothPixmapTransform);
         p.setCompositionMode(QPainter::CompositionMode_SourceIn);
@@ -374,7 +339,9 @@ QPixmap HighLightEffect::hoverGeneratePixmap(const QPixmap &pixmap, const QStyle
 
 QPixmap HighLightEffect::bothOrdinaryAndHoverGeneratePixmap(const QPixmap &pixmap, const QStyleOption *option, const QWidget *widget, EffectMode mode)
 {
-    if (!isPixmapPureColor(pixmap))
+    if (pixmap.isNull())
+        return pixmap;
+    if (!isPixmapPureColor(pixmap) || !(option->state & QStyle::State_Enabled))
         return pixmap;
 
     QPixmap target = pixmap;
@@ -386,22 +353,21 @@ QPixmap HighLightEffect::bothOrdinaryAndHoverGeneratePixmap(const QPixmap &pixma
         hoverColor = widget->property("setIconHighlightEffectHoverColor").value<QColor>();
     }
 
-    bool isEnable = option->state.testFlag(QStyle::State_Enabled);
+    if (widget && widget->property("iconHighlightEffectMode").isValid()) {
+        mode = EffectMode(widget->property("iconHighlightEffectMode").toBool());
+    }
+
     bool overOrDown =  option->state.testFlag(QStyle::State_MouseOver) ||
             option->state.testFlag(QStyle::State_Sunken) ||
             option->state.testFlag(QStyle::State_On) ||
             option->state.testFlag(QStyle::State_Selected);
-    if (auto button = qobject_cast<const QAbstractButton *>(widget)) {
-        if (button->isDown() || button->isChecked())
-            overOrDown = true;
-    }
     if (qobject_cast<const QAbstractItemView *>(widget)) {
         if (!option->state.testFlag(QStyle::State_Selected))
             overOrDown = false;
     }
 
     QPainter p(&target);
-    if (isEnable && overOrDown) {
+    if (overOrDown) {
         p.setRenderHint(QPainter::Antialiasing);
         p.setRenderHint(QPainter::SmoothPixmapTransform);
         p.setCompositionMode(QPainter::CompositionMode_SourceIn);
@@ -410,13 +376,15 @@ QPixmap HighLightEffect::bothOrdinaryAndHoverGeneratePixmap(const QPixmap &pixma
         p.setRenderHint(QPainter::Antialiasing);
         p.setRenderHint(QPainter::SmoothPixmapTransform);
         p.setCompositionMode(QPainter::CompositionMode_SourceIn);
-        p.fillRect(target.rect(), defaultColor.isValid() ? defaultColor : (mode ? defaultStyleDark() : option->palette.text()));
+        p.fillRect(target.rect(), defaultColor.isValid() ? defaultColor : (mode ? option->palette.text() : defaultStyleDark()));
     }
     return target;
 }
 
 QPixmap HighLightEffect::filledSymbolicColoredGeneratePixmap(const QPixmap &pixmap, const QStyleOption *option, const QWidget *widget, EffectMode mode)
 {
+    if (pixmap.isNull())
+        return pixmap;
     if (isPixmapPureColor(pixmap))
         return bothOrdinaryAndHoverGeneratePixmap(pixmap, option, widget, mode);
 
@@ -429,15 +397,14 @@ QPixmap HighLightEffect::filledSymbolicColoredGeneratePixmap(const QPixmap &pixm
         hoverColor = widget->property("setIconHighlightEffectHoverColor").value<QColor>();
     }
 
+    if (widget && widget->property("iconHighlightEffectMode").isValid()) {
+        mode = EffectMode(widget->property("iconHighlightEffectMode").toBool());
+    }
     bool isEnable = option->state.testFlag(QStyle::State_Enabled);
     bool overOrDown =  option->state.testFlag(QStyle::State_MouseOver) ||
             option->state.testFlag(QStyle::State_Sunken) ||
             option->state.testFlag(QStyle::State_On) ||
             option->state.testFlag(QStyle::State_Selected);
-    if (auto button = qobject_cast<const QAbstractButton *>(widget)) {
-        if (button->isDown() || button->isChecked())
-            overOrDown = true;
-    }
     if (qobject_cast<const QAbstractItemView *>(widget)) {
         if (!option->state.testFlag(QStyle::State_Selected))
             overOrDown = false;
@@ -446,7 +413,7 @@ QPixmap HighLightEffect::filledSymbolicColoredGeneratePixmap(const QPixmap &pixm
     if (isEnable && overOrDown) {
         return filledSymbolicColoredPixmap(target, hoverColor.isValid() ? hoverColor : option->palette.color(QPalette::HighlightedText));
     } else {
-        return filledSymbolicColoredPixmap(target, defaultColor.isValid() ? defaultColor : (mode ? defaultStyleDark() : option->palette.color(QPalette::Text)));
+        return filledSymbolicColoredPixmap(target, defaultColor.isValid() ? defaultColor : (mode ? option->palette.color(QPalette::Text) : defaultStyleDark()));
     }
 
     return target;
@@ -459,18 +426,19 @@ HighLightEffect::HighLightEffect(QObject *parent) : QObject(parent)
 
 QPixmap HighLightEffect::filledSymbolicColoredPixmap(const QPixmap &source, const QColor &baseColor)
 {
+    if (source.isNull())
+        return source;
     QImage img = source.toImage();
     for (int x = 0; x < img.width(); x++) {
         for (int y = 0; y < img.height(); y++) {
             auto color = img.pixelColor(x, y);
-            if (color.alpha() > 0) {
-                int hue = color.hue();
-                if (qAbs(hue - symbolic_color.hue()) < 10) {
+            if (color.alpha() > 0.3) {
+                if (qAbs(color.red() - symbolic_color.red()) < 10 && qAbs(color.green() - symbolic_color.green()) < 10
+                        && qAbs(color.blue() - symbolic_color.blue()) < 10) {
                     color.setRed(baseColor.red());
                     color.setGreen(baseColor.green());
                     color.setBlue(baseColor.blue());
                     img.setPixelColor(x, y, color);
-                    qDebug()<<"fill"<<color<<baseColor;
                 }
             }
         }
